@@ -3,7 +3,6 @@ from gql.client import RetryError  # type: ignore
 from gql.transport.requests import RequestsHTTPTransport  # type: ignore
 import datetime
 import os
-import requests
 import ast
 import os
 import json
@@ -329,6 +328,7 @@ class Api(object):
             viewer {
                 id
                 entity
+                flags
                 teams {
                     edges {
                         node {
@@ -975,7 +975,7 @@ class Api(object):
             A tuple of the file's local path and the streaming response. The streaming response is None if the file already existed and was up to date.
         """
         fileName = metadata['name']
-        path = os.path.join(out_dir or wandb_dir(), fileName)
+        path = os.path.join(out_dir or self.settings('wandb_dir'), fileName)
         if self.file_current(fileName, metadata['md5']):
             return path, None
 
@@ -1424,6 +1424,10 @@ class Api(object):
                     id
                     digest
                     state
+                    aliases {
+                        artifactCollectionName
+                        alias
+                    }
                 }
             }
         }
@@ -1447,6 +1451,11 @@ class Api(object):
             'metadata': json.dumps(util.make_safe_for_json(metadata)) if metadata else None,
         })
         av = response['createArtifact']['artifact']
+        # TODO: make this a part of the graph 
+        av['version'] = "latest"
+        for alias in av["aliases"]:
+            if alias["artifactCollectionName"] == artifact_collection_name and re.match(r"^v\d+$", alias["alias"]):
+                av['version'] = alias["alias"]
         return av
 
     def commit_artifact(self, artifact_id):
