@@ -72,7 +72,7 @@ defaults = dict(
     _mode=Field(str, ("auto", "noop", "online", "offline", "dryrun", "run",)),
     _problem=Field(str, ("fatal", "warn", "silent",)),
     console="auto",
-    _console=Field(str, ("auto", "redirect", "off", "mock", "file", "iowrap",)),
+    _console=Field(str, ("auto", "redirect", "off", "file", "iowrap",)),
     git_remote="origin",
 )
 
@@ -95,6 +95,7 @@ env_settings = dict(
     host=None,
     username=None,
     disable_code=None,
+    wandb_dir="WANDB_DIR",
     run_name="WANDB_NAME",
     run_notes="WANDB_NOTES",
     run_tags="WANDB_TAGS",
@@ -152,11 +153,11 @@ def _get_program():
     if os.path.exists(full_path_to_program):
         relative_path = os.path.relpath(full_path_to_program, start=root)
         if "../" in relative_path:
-            logger.warn("could not save program above cwd: %s" % program)
+            logger.warning("could not save program above cwd: %s" % program)
             return None
         return relative_path
 
-    logger.warn("could not find program at %s" % program)
+    logger.warning("could not find program at %s" % program)
     return None
 
 
@@ -500,7 +501,11 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
     def apply_init(self, args):
         # strip out items where value is None
         param_map = dict(
-            name="run_name", id="run_id", tags="run_tags", group="run_group",
+            name="run_name",
+            id="run_id",
+            tags="run_tags",
+            group="run_group",
+            dir="wandb_dir",
         )
         args = {param_map.get(k, k): v for k, v in six.iteritems(args) if v is not None}
         self.update(args)
